def sinh(x):
    return (e**x - e**(-x))/2
    
def cosh(x):
    return (e**x + e**(-x))/2
    
def tanh(x):
    return sinh(x) / cosh(x)
    
def coth(x):
    return cosh(x) / sinh(x)
    
def sech(x):
    return 1 / cosh(x)
    
def cosech(x):
    return 1 / sinh(x)
    
def arcsinh(x):
    return ln(x + sqrt(x**2 + 1))
    
def arccosh(x):
    return abs(arcsinh(sqrt(x**2 - 1)))
    
def arccoth(x):
    return 0.5 * ((x+1)/(x-1))
    
def arctanh(x):
    return arccoth(1/x)
    
def arcsech(x):
    return arccosh(1/x)
    
def arccosech(x):
    return arcsinh(1/x)


# integrating integrals
def integrate(function, lowlim, uplim, n=10000):
    h = (uplim - lowlim) / n
    s = function(lowlim) + function(uplim)

    for i in range(1, n):
        x = lowlim + i * h
        if i % 2 == 0:
            s += 2 * function(x)
        else:
            s += 4 * function(x)

    return s * h / 3
    
def summation(lowlim, uplim, function, max_terms=10**6):
    total = 0
    count = 0
    
    if uplim == float("inf"):
        n = lowlim
        while count < max_terms:
            try:
                term = function(n)
            except ZeroDivisionError:
                n += 1
                continue
            
            total += term
            
            # Stop if terms are negligible
            if abs(term) < 1e-10:
                break
                
            n += 1
            count += 1
        
        return total
    
    else:
        for n in range(int(lowlim), int(uplim) + 1):
            total += function(n)
        return total
        
def converges(f, tolerance=1e-6, max_terms=10**6):
    total = 0
    prev = 0
    
    for n in range(1, max_terms):
        try:
            total += f(n)
        except ZeroDivisionError:
            continue
        
        # Check if partial sum stabilizes
        if abs(total - prev) < tolerance:
            return True
        
        prev = total
    
    return False
    
def lim(f, xlim, tol=1e-5):
    # Try direct substitution
    try:
        return f(xlim)
    except ZeroDivisionError:
        pass  # expected for limits
    except Exception as e:
        return None  # or raise e if you want strict behavior

    hs = [1e-1, 1e-2, 1e-3, 1e-4, 1e-5]
    left_vals = []
    right_vals = []

    for h in hs:
        try:
            left_vals.append(f(xlim - h))
        except ZeroDivisionError:
            continue

        try:
            right_vals.append(f(xlim + h))
        except ZeroDivisionError:
            continue

    if not left_vals or not right_vals:
        return None

    left = left_vals[-1]
    right = right_vals[-1]

    if abs(left - right) < tol:
        return (left + right) / 2

    return None


from AxiomX.constants import *
from AxiomX.functions import *

def radians(deg):
    return (pi/180)*deg
    
def degrees(rad):
    return (180/pi)*rad
    
def sin(x, terms=20):
    quarter = ((x // tau) + 1) % 4
    if x == pi:
        return 0
    x = x % tau
    # Input validation
    if not isinstance(x, (int, float)):
        raise TypeError("x must be a number (int or float).")
    if not isinstance(terms, int) or terms <= 0:
        raise ValueError("terms must be a positive integer.")
    sine_value = 0.0
    for n in range(terms):
        term = ((-1)**n) * (x**(2*n + 1)) / factorial(2*n + 1)
        sine_value += term
    if quarter == 1:
        return sine_value
    elif quarter == 2:
        return sqrt(1 - (sine_value**2))
    elif quarter == 3:
        return -sine_value
    elif quarter == 0:
        return -sqrt(1 - (sine_value**2))
        
def cos(x):
    return sin((pi/2)-x)
    
def tan(x):
    if cos(x) == 0:
        return float("inf")
    return sin(x) / cos(x)

def cot(x):
    if tan(x) == 0:
        return float("inf")
    return 1 / tan(x)

def sec(x):
    if cos(x) == 0:
        return float("inf")
    return 1 / cos(x)
    
def cosec(x):
    if sin(x) == 0:
        return float("inf")
    return 1 / sin(x)

def arcsin(x, iterations=10):
    if abs(x) > 1:
        raise ValueError("x must be in [-1, 1]")
    y = x
    for _ in range(iterations):
        if cos(y) == 0:
            break
        y -= (sin(y) - x) / cos(y)
    return y
    
def arccos(x):
    return (pi / 2) - arcsin(x)
    
def arctan(x):
    return arcsin(x / sqrt(1+ x**2))
    
def arccot(x):
    return (pi/2) - arctan(x)

def arcsec(x):
    if x < 1:
        raise ValueError("x < 1")
    return arccos(1/x)
    
def arccosec(x):
    if x < 1:
        raise ValueError("x < 1")
    return arcsin(1/x)


def exp(n):
    return e**n

def ln(x):
    if x <= 0:
        raise ValueError("ln(x) is undefined for x <= 0")
    k = 0
    while x >= 2.0:
        x *= 0.5
        k += 1
    while x < 1.0:
        x *= 2.0
        k -= 1
    y = (x - 1) / (x + 1)
    y2 = y * y
    s = 0.0
    term = y
    n = 1
    while abs(term) > 1e-17:
        s += term / n
        term *= y2
        n += 2
    return 2*s + k * (0.693147180559945309417232121458176568) # ln 2
    
def log10(x):
    return ln(x) / ln(10)
    
def log2(x):
    return log10(x) / log10(2)
    
def log(arg, base):
    return log2(arg) / log2(base)


def absolute(x):
    if (x > 0):
        return x
    else:
        return -x
        
def sqrt(a, x0=None, tol=1e-30, max_iter=20):
    if a < 0:
        raise ValueError("a must be non-negative")

    if a == 0:
        return 0.0

    # Initial guess
    x = a if x0 is None else x0

    for _ in range(max_iter):
        x2 = x * x
        x_new = x * (x2 + 3*a) / (3*x2 + a)

        if abs(x_new - x) < tol * x_new:
            return x_new

        x = x_new

    return x


def cbrt(N, tolerance=1e-10, max_iterations=1000):
    x = N / 3.0 if N != 0 else 0.0
    for i in range(max_iterations):
        x_next = x - (x**3 - N) / (3 * x**2)
        if abs(x_next - x) < tolerance:
            return x_next
        x = x_next 
    return x

def gamma(x):
    # Lanczos approximation constants
    p = [
        0.99999999999980993,
        676.5203681218851,
        -1259.1392167224028,
        771.32342877765313,
        -176.61502916214059,
        12.507343278686905,
        -0.13857109526572012,
        9.9843695780195716e-6,
        1.5056327351493116e-7
    ]

    if x < 0.5:
        # Reflection formula
        return pi / (sin(pi * x) * gamma(1 - x))

    x -= 1
    t = p[0]
    for i in range(1, len(p)):
        t += p[i] / (x + i)

    g = 7
    return sqrt(2 * pi) * (x + g + 0.5)**(x + 0.5) * (e**(-(x + g + 0.5))) * t
    
def factorial(x):
    if (x // 1 == x):
        f = 1
        while x > 0:
            f *= x
            x -= 1
        return f
    else:
        return gamma(x+1)

def zeta(n):
    zetval = 0
    if n <= 1:
        raise ValueError("zeta(n) diverges for n <= 1")
    for _ in range(1, 100001):
        zetval += (1 / _**n)
    return zetval
    
def beta(n):
    if n == 0:
        return 0.5
    total = 0.0
    for i in range(100000):
        total += ((-1)**i) / ((2*i + 1)**n)
    return total
    
def bc(n, k):
    if k < 0 or k > n:
        raise ValueError("Binomial coefficient (n k) doesn't work for negative n or k, or if n < k")
    
    k = min(k, n - k)  # symmetry optimization
    result = 1
    
    for i in range(1, k + 1):
        result = result * (n - i + 1) // i
    
    return result
        
def pascal(row):
    pr = []
    for i in range(0, row+1):
        pr.append(int(bc(row, i)))
    return pr


import time

print("AxiomX loading...")
time.sleep(3)
print("AxiomX is ready to use!")

import AxiomX.constants
import AxiomX.calculus
import AxiomX.functions
import AxiomX.exp
import AxiomX.trig
import AxiomX.hyperbolic

version = '0.1.0'



pi = 3.141592653589793
e = 2.718281828459045
tau = 2*pi
lemniscate = 2.622057554292119
gauss = lemniscate / pi
euler_mascheroni = 0.577215664901533
sqrt_2 = 2**0.5
sqrt_3 = 3**0.5
sqrt_5 = 5**0.5
golden_ratio = (1 + sqrt_5) / 2
silver_ratio = 1 + sqrt_2
gelfond = e**pi
gelfond_schneider = 2**sqrt_2
infinity = float("inf")

def metallic_ratio(n):
    return (n + (n**2 + 4)**0.5) / 2